@@ -1,5 +1,5 @@
 ;;  -*-  indent-tabs-mode:nil; coding: utf-8 -*-
-;;  Copyright (C) 2016,2017
+;;  Copyright (C) 2016,2017,2018
 ;;      "Mu Lei" known as "NalaGinrut" <NalaGinrut@gmail.com>
 ;;  Artanis is free software: you can redistribute it and/or modify
 ;;  it under the terms of the GNU General Public License and GNU
@@ -39,7 +39,7 @@
 ;; on what purpose associated file descriptor serves.
 (define-public EPOLLPRI #x002) ; There is urgent data available for read operations.
 
-(define-public EPOLLOUT #x004) ; The associated file is available for write operations. 
+(define-public EPOLLOUT #x004) ; The associated file is available for write operations.
 (define-public EPOLLWRNORM #x100) ; identical to EPOLLOUT
 
 ;; on some sockets there will be the data send with MSG_OOB flag passed to socket.
@@ -77,6 +77,15 @@
 ;; Set exclusive wakeup mode for the target file descriptor.
 (define-public EPOLLEXCLUSIVE (ash 1 28))
 
+(define-public *error-event* (logior EPOLLRDHUP EPOLLHUP))
+(define-public *read-event* EPOLLIN)
+(define-public (gen-read-event) (logior *read-event* (get-trigger)))
+(define-public *rw-event* (logior EPOLLIN EPOLLOUT))
+(define-public (gen-rw-event) (logior *error-event* (get-trigger) *rw-event*))
+(define-public *write-event* EPOLLOUT)
+(define-public (gen-write-event) (logior *error-event* (get-trigger) *write-event*))
+(define-public (gen-oneshot-event) (logior EPOLLONESHOT (gen-rw-event)))
+
 (define-public epoll-data-meta (list '* int uint32 uint64))
 (define-public epoll-data-size (sizeof epoll-data-meta))
 
@@ -187,7 +196,7 @@
                       (dynamic-func "epoll_ctl" (dynamic-link))
                       (list int int int '*) #:return-errno? #t))
 
-(define* (epoll-ctl epfd op fd event #:key (keep-alive? #f))
+(define* (epoll-ctl epfd op fd event #:key (check-exists? #f))
   (call-with-values
       (lambda ()
         (%epoll-ctl epfd op fd (if event
@@ -196,15 +205,23 @@
     (lambda (ret errno)
       (cond
        ((zero? ret) ret)
-       ((and (= ret EEXIST) keep-alive?)
+       (check-exists?
         (DEBUG "The event ~a exist and kept alive~%" fd)
-        0)
+        (= ret EEXIST))
        (else
         (throw 'artanis-err 500 epoll-ctl "~a: ~a"
                (list epfd op fd event (strerror errno))
                (list errno)))))))
 (export epoll-ctl)
 
+(define-public (exists-in-epoll? epfd fd)
+  (let* ((ees (make-epoll-event-set))
+         (ret (epoll-ctl epfd EPOLL_CTL_ADD fd ees #:check-exists? #t)))
+    (when (not ret)
+      ;; Must remove it after test
+      (epoll-ctl epfd EPOLL_CTL_DEL fd #f))
+    ret))
+
 ;; NOTE: do NOT use this function outside this module!!!
 (define (epoll-event-set->list ees nfds)
   (let lp((i 0) (ret '()))
@@ -235,6 +252,7 @@
       (lambda (ret errno)
         (cond
          ((>= ret 0) (epoll-event-set->list events ret))
+         ((= errno EINTR) (epoll-wait epfd events timeout))
          (else
           (throw 'artanis-err 500 epoll-wait "~a: ~a"
                  (list epfd events maxevents timeout (strerror errno))
@@ -264,12 +282,17 @@
   (if (logtest (cdr e) EPOLLRDHUP)
       (if (logtest (cdr e) EPOLLIN)
           (begin
-            (DEBUG "Peer ~a is half-closed, but we can still read, close it next time!~%"
+            (DEBUG "Peer ~a is half-read, but we can still read, close it next time!~%"
                    e)
-            #f) ; yes peer is shutdown but we return false for reading more data
-          (begin
-            (DEBUG "Peer ~a is completely closed!~%" e)
-            #t)) ; yes peer shutdown is true
+            'half-read) ; yes peer is shutdown but we return false for reading more data
+          (if (logtest (cdr e) EPOLLOUT)
+              (begin
+                (DEBUG "Peer ~a is half-write, but we can still read, close it next time!~%"
+                       e)
+                'half-write)
+              (begin
+                (DEBUG "Peer ~a is completely closed!~%" e)
+                #t))) ; yes peer shutdown is true
       (begin
         (DEBUG "Peer ~a is still alive!~%" e)
-        #f))) ; no peer is not shutdown
+        #f))) ; peer is not shutdown
